from .main import *
from seqeval.metrics import accuracy_score, classification_report


class ncrfppModel:
    def __init__(self, config):
        self.data_config = Data()
        self.config = config

    def set_config(self, dset=None, data=None, custom_config=None):
        if dset is not None:
            self.data_config.load(dset)
        if data is None:
            self.data_config.read_config(self.config)
        else:
            self.data_config = data
        if custom_config is not None:
            self.data_config.read_config(self.config, custom_config)

    def get_config(self):
        return self.data_config

    def train(self, log='test.log', metric='F'):
        status = self.data_config.status.lower()
        if status == 'train':
            print("MODEL: train")
            data_initialization(self.data_config)
            self.data_config.generate_instance('train')
            self.data_config.generate_instance('dev')
            self.data_config.generate_instance('test')
            self.data_config.build_pretrain_emb()
            self.data_config.summary()
            train(self.data_config, log, metric)

    def decode(self):
        status = self.data_config.status.lower()

        if status == 'decode':
            print("MODEL: decode")
            self.data_config.generate_instance('raw')
            print("nbest: {}".format(self.data_config.nbest))
            decode_results, pred_scores = load_model_decode(self.data_config, 'raw')
            if self.data_config.nbest > 0 and not self.data_config.sentence_classification:
                self.data_config.write_nbest_decoded_results(decode_results, pred_scores, 'raw')
            else:
                self.data_config.write_decoded_results(decode_results, 'raw')

    def get_gold_predict(self, golden_standard, predict_result, stoken):
        golden_data = open(golden_standard, 'r', encoding='utf-8').readlines()
        predict_data = open(predict_result, 'r', encoding='utf-8').readlines()
        golden_list = []
        predict_list = []
        tmp_gold = []
        tmp_predict = []
        for gold_idx, pre_idx in zip(golden_data, predict_data):
            if gold_idx != '\n':
                gentity_with_label = gold_idx.split(stoken)
                glabel = gentity_with_label[1].replace('\n', '')
                tmp_gold.append(glabel)
                pentity_with_label = pre_idx.split(stoken)
                plabel = pentity_with_label[1].replace('\n', '')
                tmp_predict.append(plabel)
            else:
                golden_list.append(tmp_gold)
                predict_list.append(tmp_predict)
                tmp_gold = []
                tmp_predict = []
        return golden_list, predict_list

    def report_f1(self, golden_standard, predict_result, split=" "):
        golden_list, predict_list = self.get_gold_predict(golden_standard, predict_result, split)
        print(classification_report(golden_list, predict_list))

    def report_acc(self, golden_standard, predict_result, split=' ||| '):
        golden_list, predict_list = self.get_gold_predict(golden_standard, predict_result, split)
        print("Report accuracy: %0.2f" % accuracy_score(golden_list, predict_list))
